==> clickhouse-0.11.0.tar: lib/clickhouse/result.ex <==
defmodule ClickHouse.Result do
  @moduledoc """
  The results of a ClickHouse query.
  """

  alias ClickHouse.Config

  @derive {Inspect, only: [:data, :columns]}
  @enforce_keys [:client, :raw, :raw_format, :meta, :compressed]
  defstruct [:client, :raw, :raw_format, :format, :columns, :meta, :compressed, :data]

  @typedoc """
  The name of the column.
  """
  @type column_name :: binary()

  @typedoc """
  The data type of the column.
  """
  @type column_type ::
          :i64
          | :i32
          | :i16
          | :i8
          | :u64
          | :u32
          | :u16
          | :u8
          | :f64
          | :f32
          | :string
          | :uuid
          | :date
          | :datetime
          | {:fixed_string, integer()}
          | {:enum8, [binary()]}
          | {:enum16, [binary()]}
          | {:array, column_type()}
          | {:low_cardinality, column_type()}
          | {:nullable, column_type()}

  @typedoc """
  The names and data types of columns.
  """
  @type columns :: [{column_name(), column_type()}, ...] | nil

  @typedoc """
  The data returned from ClickHouse.
  """
  @type data :: [] | [list(), ...] | nil

  @typedoc """
  The result of a ClickHouse query.
  """
  @type t :: %__MODULE__{
          client: ClickHouse.client(),
          raw: binary(),
          raw_format: binary(),
          format: ClickHouse.Format.t(),
          columns: columns(),
          data: data(),
          compressed: boolean(),
          meta: map()
        }

  ################################
  # Public API
  ################################

  @spec new(ClickHouse.client(), binary(), binary(), map(), boolean()) :: ClickHouse.Result.t()
  def new(client, raw, raw_format, meta, compressed) do
    %__MODULE__{
      client: client,
      raw: raw,
      raw_format: raw_format,
      meta: meta,
      compressed: compressed
    }
    |> get_format()
  end

  @spec decode(ClickHouse.Result.t()) :: ClickHouse.Result.t()
  def decode(%{compressed: true} = result), do: result
  def decode(%{format: nil} = result), do: result
  def decode(result), do: result.format.decode(result)

  ################################
  # Public API
  ################################

  defp get_format(result) do
    case Config.format(result.client, result.raw_format) do
      {:ok, format} -> %{result | format: format}
      _ -> result
    end
  end
end

==> clickhouse-0.12.1.tar: lib/clickhouse/result.ex <==
defmodule ClickHouse.Result do
  @moduledoc """
  The results of a ClickHouse query.
  """

  alias ClickHouse.Config

  @derive {Inspect, only: [:data, :columns]}
  @enforce_keys [:client, :raw, :raw_format, :meta, :compressed]
  defstruct [:client, :raw, :raw_format, :format, :columns, :meta, :compressed, :data]

  @typedoc """
  The name of the column.
  """
  @type column_name :: binary()

  @typedoc """
  The data type of the column.
  """
  @type column_type ::
          :i64
          | :i32
          | :i16
          | :i8
          | :u64
          | :u32
          | :u16
          | :u8
          | :f64
          | :f32
          | :string
          | :uuid
          | :date
          | :datetime
          | {:fixed_string, integer()}
          | {:enum8, [binary()]}
          | {:enum16, [binary()]}
          | {:array, column_type()}
          | {:low_cardinality, column_type()}
          | {:nullable, column_type()}

  @typedoc """
  The names and data types of columns.
  """
  @type columns :: [{column_name(), column_type()}, ...] | nil

  @typedoc """
  The data returned from ClickHouse.
  """
  @type data :: [] | [list(), ...] | nil

  @typedoc """
  The result of a ClickHouse query.
  """
  @type t :: %__MODULE__{
          client: ClickHouse.client(),
          raw: binary(),
          raw_format: binary(),
          format: ClickHouse.Format.t(),
          columns: columns(),
          data: data(),
          compressed: boolean(),
          meta: map()
        }

  ################################
  # Public API
  ################################

  @spec new(ClickHouse.client(), binary(), binary(), map(), boolean()) :: ClickHouse.Result.t()
  def new(client, raw, raw_format, meta, compressed) do
    %__MODULE__{
      client: client,
      raw: raw,
      raw_format: raw_format,
      meta: meta,
      compressed: compressed
    }
    |> get_format()
  end

  @spec decode(ClickHouse.Result.t()) :: ClickHouse.Result.t()
  def decode(%{compressed: true} = result), do: result
  def decode(%{format: nil} = result), do: result

  def decode(result) do
    {columns, data} = result.format.decode(result.raw)
    %{result | columns: columns, data: data}
  end

  ################################
  # Public API
  ################################

  defp get_format(result) do
    case Config.format(result.client, result.raw_format) do
      {:ok, format} -> %{result | format: format}
      _ -> result
    end
  end
end
